from datetime import datetime

import discord
from cpl_core.configuration import ConfigurationABC
from cpl_core.logging import LoggerABC
from cpl_discord.command import DiscordCommandABC
from cpl_discord.service import DiscordBotServiceABC
from cpl_translation import TranslatePipe
from discord.ext import commands
from discord.ext.commands import Context

import bot
from bot_core.abc.client_utils_service_abc import ClientUtilsServiceABC
from bot_core.abc.message_service_abc import MessageServiceABC


class InfoCommand(DiscordCommandABC):

    def __init__(
            self,
            config: ConfigurationABC,
            logger: LoggerABC,
            message_service: MessageServiceABC,
            bot: DiscordBotServiceABC,
            client_utils: ClientUtilsServiceABC,
            translate: TranslatePipe
    ):
        DiscordCommandABC.__init__(self)

        self._config = config
        self._logger = logger
        self._message_service = message_service
        self._bot = bot
        self._client_utils = client_utils
        self._t = translate

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    @commands.guild_only()
    async def info(self, ctx: Context, *, wait: int = None):
        self._logger.debug(__name__, f'Received command info {ctx},{wait}')
        self._client_utils.received_command(ctx.guild.id)
        client = self._client_utils.get_client(self._bot.user.id, ctx.guild.id)

        embed = discord.Embed(
            title=self._t.transform('modules.base.info.title'),
            description=self._t.transform('modules.base.info.description'),
            color=int('ef9d0d', 16)
        )

        embed.add_field(name=self._t.transform('modules.base.info.fields.version'), value=bot.__version__)
        start_time = self._config.get_configuration('Bot_StartTime')
        ontime = round((datetime.now() - datetime.strptime(start_time, '%Y-%m-%d %H:%M:%S.%f')).total_seconds() / 3600, 2)
        embed.add_field(name=self._t.transform('modules.base.info.fields.ontime'), value=f'{ontime}h')
        embed.add_field(name=self._t.transform('modules.base.info.fields.sent_message_count'), value=client.sent_message_count, inline=False)
        embed.add_field(name=self._t.transform('modules.base.info.fields.received_message_count'), value=client.received_message_count)
        embed.add_field(name=self._t.transform('modules.base.info.fields.deleted_message_count'), value=client.deleted_message_count, inline=False)
        embed.add_field(name=self._t.transform('modules.base.info.fields.received_command_count'), value=client.received_command_count)
        embed.add_field(name=self._t.transform('modules.base.info.fields.moved_users_count'), value=client.moved_users_count)
        modules = ['Base', 'BootLog', 'Database', 'Permission']
        embed.add_field(name=self._t.transform('modules.base.info.fields.modules'), value='\n'.join(modules), inline=False)

        await self._message_service.send_ctx_msg(ctx, embed, wait_before_delete=wait)
        self._logger.trace(__name__, f'Finished info command')
